require 'fastlane/action'
require_relative '../app/app_main'
require_relative '../tools/message'
module Fastlane
  module Actions
    class AppAction < Action
      def self.run(params)

        begin
          App::AppMain.run(params,available_options)
        rescue Exception => e
          Tools.error(e.message)
        end 
        Tools.title("完成!!!")
      end

      def self.description
        "APP打包"
      end

      def self.authors
        ["三块"]
      end

      def self.return_value
        # If your method provides a return value, you can describe here what it does
      end

      def self.details
        "开发者只能打开发者包，App Store包需要移步至rocket平台"
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :code,
                                       env_name: 'R_APP_CODE',
                                       description: '应用的标识码，从rocket平台获取。用于获取应用的相关信息',
                                       optional: false,
                                       type: String,
                                       default_value: "fanDengBook",
                                       verify_block: proc do |value| 
                                         Tools.error("code不能为空") unless (value and not value.empty?)
                                         ENV["R_APP_CODE"] = value.to_s
                                       end),
          FastlaneCore::ConfigItem.new(key: :plan,
                                       env_name: 'R_APP_PLAN',
                                       description: '测试计划或发布计划。用于确定组件的版本和打包信息',
                                       optional: false,
                                       type: String,
                                       verify_block: proc do |value|
                                         Tools.error("plan不能为空") unless (value and not value.empty?)
                                         ENV["R_APP_PLAN"] = value.to_s
                                       end),
          FastlaneCore::ConfigItem.new(key: :config,
                                       env_name: 'R_APP_CONFIG',
                                       description: '打包环境:Debug/Release',
                                       optional: false,
                                       type: String,
                                       default_value: "Debug",
                                       verify_block: proc do |value|
                                         Tools.error("config不能为空") unless (value and not value.empty?)
                                         Tools.error("config没有#{value}选项") unless (value == "Debug" or value == "Release")
                                         ENV["R_APP_CONFIG"] = value.to_s
                                       end),
          FastlaneCore::ConfigItem.new(key: :channel,
                                       env_name: 'R_APP_CHANNEL',
                                       description: '打包方式:APP_STORE/DEVELOPMENT',
                                       optional: false,
                                       type: String,
                                       default_value: "DEVELOPMENT",
                                       verify_block: proc do |value|
                                         Tools.error("channel不能为空") unless (value and not value.empty?)
                                         ENV["R_APP_CHANNEL"] = value.to_s
                                       end),
          FastlaneCore::ConfigItem.new(key: :build,
                                       env_name: 'R_APP_BUILD',
                                       description: 'build number',
                                       optional: false,
                                       type: Integer,
                                       verify_block: proc do |value|
                                         ENV["R_APP_BUILD"] = value.to_s
                                       end),
          FastlaneCore::ConfigItem.new(key: :note,
                                       env_name: 'R_APP_NOTE',
                                       description: '打包日志',
                                       optional: false,
                                       type: String,
                                       verify_block: proc do |value|
                                         ENV["R_APP_NOTE"] = value.to_s
                                       end),

        ]
      end

      def self.is_supported?(platform)
        # Adjust this if your plugin only works for a particular platform (iOS vs. Android, for example)
        # See: https://docs.fastlane.tools/advanced/#control-configuration-by-lane-and-by-platform
        #
        # [:ios, :mac, :android].include?(platform)
        true
      end
    end
  end
end
