require 'fastlane/action'
require_relative '../ref/ref_main'
require_relative '../tools/message'
module Fastlane
  module Actions
    class RefAction < Action
      def self.run(params)
        begin
          Ref::RefMain.run(params,available_options)
        rescue Exception => e
          Tools.error(e.message)
        end 
        Tools.title("完成!!!")
      end

      def self.description
        "查找引入文件，却未在podsepc依赖文件的来源库"
      end

      def self.authors
        ["三块"]
      end

      def self.return_value
        # If your method provides a return value, you can describe here what it does
      end

      def self.details
        "生成表，表明哪些库需要在哪些podspec中引入"
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :git,
                                       env_name: 'R_REF_GIT',
                                       description: '工程仓库地址',
                                       optional: false,
                                       type: String,
                                       verify_block: proc do |value| 
                                         Tools.error("git不能为空") unless (value and not value.empty?)
                                         ENV["R_REF_GIT"] = value.to_s
                                       end),
          FastlaneCore::ConfigItem.new(key: :branch,
                                       env_name: 'R_REF_BRANCH',
                                       description: '工程分支',
                                       optional: false,
                                       type: String,
                                       verify_block: proc do |value|
                                         Tools.error("branch不能为空") unless (value and not value.empty?)
                                         ENV["R_REF_BRANCH"] = value.to_s
                                       end),
          FastlaneCore::ConfigItem.new(key: :pods,
                                       env_name: 'R_REF_PODS',
                                       description: '默认全量查询(耗时很久请勿使用),需要查询的pod组合，逗号分隔。1.找出哪些库使用该pod组件，却未在podsepc依赖库 2.分析import方式错误的依赖库',
                                       optional: true,
                                       default_value:"ALL",
                                       type: String,
                                       verify_block: proc do |value|
                                         Tools.error("pods不能为空") unless (value and not value.empty?)
                                         ENV["R_REF_PODS"] = value.to_s
                                       end)

        ]
      end

      def self.is_supported?(platform)
        # Adjust this if your plugin only works for a particular platform (iOS vs. Android, for example)
        # See: https://docs.fastlane.tools/advanced/#control-configuration-by-lane-and-by-platform
        #
        # [:ios, :mac, :android].include?(platform)
        true
      end
    end
  end
end
